@@ -33,6 +33,7 @@
 #include <unistd.h>
 #include <errno.h>
 #include <time.h>
+#include <stdlib.h>
 
 #if !HAVE_O_LARGEFILE
 #define O_LARGEFILE 0
@@ -95,6 +96,16 @@ affile_open_file(gchar *name, int flags,
   return *fd != -1;
 }
 
+static inline gchar *
+affile_sd_format_persist_name(AFFileSourceDriver *self)
+{
+  static gchar persist_name[1024];
+  
+  g_snprintf(persist_name, sizeof(persist_name), "affile_sd_curpos(%s)", self->filename->str);
+  return persist_name;
+}
+
+
 static gboolean
 affile_sd_init(LogPipe *s, GlobalConfig *cfg, PersistentConfig *persist)
 {
@@ -111,6 +122,21 @@ affile_sd_init(LogPipe *s, GlobalConfig *cfg, PersistentConfig *persist)
   if (affile_open_file(self->filename->str, flags, -1, -1, -1, 0, 0, 0, 0, &fd))
     {
       self->reader = log_reader_new(fd_read_new(fd, 0), LR_LOCAL | LR_NOMREAD, s, &self->reader_options);
+
+      if (persist)
+        {
+          gchar *str;
+          off_t cur_pos;
+          
+          str = persist_config_fetch(persist, affile_sd_format_persist_name(self));
+          if (str)
+            {
+              cur_pos = strtoll(str, NULL, 10);
+              log_reader_set_pos((LogReader *) self->reader, cur_pos);
+              g_free(str);
+            }
+          
+        }
       log_pipe_append(self->reader, s);
       if (!log_pipe_init(self->reader, NULL, NULL))
         {
@@ -139,6 +165,16 @@ affile_sd_deinit(LogPipe *s, GlobalConfig *cfg, PersistentConfig *persist)
 {
   AFFileSourceDriver *self = (AFFileSourceDriver *) s;
 
+  if (persist)
+    {
+      gchar str[32];
+      off_t cur_pos;
+      
+      cur_pos = log_reader_get_pos((LogReader *) self->reader);
+      g_snprintf(str, sizeof(str), "%" G_GINT64_FORMAT, (gint64) cur_pos);
+          
+      persist_config_add_survivor(persist, affile_sd_format_persist_name(self), str);
+    }
   if (self->reader)
     {
       log_pipe_deinit(self->reader, NULL, NULL);
@@ -206,8 +242,9 @@ affile_dw_init(LogPipe *s, GlobalConfig *cfg, PersistentConfig *persist)
       stat(self->filename->str, &st) == 0 && 
       st.st_mtime < time(NULL) - self->owner->overwrite_if_older)
     {
-      msg_notice("Destination file is too old, removing",
+      msg_info("Destination file is older than overwrite_if_older(), overwriting",
                  evt_tag_str("filename", self->filename->str),
+                 evt_tag_int("overwrite_if_older", self->owner->overwrite_if_older),
                  NULL);
       unlink(self->filename->str);
     }
@@ -310,18 +347,6 @@ affile_dw_new(AFFileDestDriver *owner, GString *filename)
   return &self->super;
 }
 
-void 
-affile_dd_set_compress(LogDriver *s, gboolean compress G_GNUC_UNUSED)
-{
-  msg_error("On-the-fly file compression is not supported", NULL);
-}
-
-void 
-affile_dd_set_encrypt(LogDriver *s, gboolean compress G_GNUC_UNUSED)
-{
-  msg_error("On-the-fly file encryption is not supported", NULL);
-}
-
 void 
 affile_dd_set_file_uid(LogDriver *s, const gchar *file_uid)
 {
@@ -330,7 +355,7 @@ affile_dd_set_file_uid(LogDriver *s, const gchar *file_uid)
   self->file_uid = 0;
   if (!resolve_user(file_uid, &self->file_uid))
     {
-      msg_notice("Error resolving user",
+      msg_error("Error resolving user",
                  evt_tag_str("user", file_uid),
                  NULL);
     }
@@ -344,7 +369,7 @@ affile_dd_set_file_gid(LogDriver *s, const gchar *file_gid)
   self->file_gid = 0;
   if (!resolve_group(file_gid, &self->file_gid))
     {
-      msg_notice("Error resolving group",
+      msg_error("Error resolving group",
                  evt_tag_str("group", file_gid),
                  NULL);
     }
@@ -366,7 +391,7 @@ affile_dd_set_dir_uid(LogDriver *s, const gchar *dir_uid)
   self->dir_uid = 0;
   if (!resolve_user(dir_uid, &self->dir_uid))
     {
-      msg_notice("Error resolving user",
+      msg_error("Error resolving user",
                  evt_tag_str("user", dir_uid),
                  NULL);
     }
@@ -380,7 +405,7 @@ affile_dd_set_dir_gid(LogDriver *s, const gchar *dir_gid)
   self->dir_gid = 0;
   if (!resolve_group(dir_gid, &self->dir_gid))
     {
-      msg_notice("Error resolving group",
+      msg_error("Error resolving group",
                  evt_tag_str("group", dir_gid),
                  NULL);
     }
