@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2003, 2004 BalaBit IT Ltd, Budapest, Hungary
+ * Copyright (c) 2002-2007 BalaBit IT Ltd, Budapest, Hungary                    
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -26,13 +26,26 @@
 
 #include "afsocket.h"
 
+#if ENABLE_SPOOF_SOURCE
+#include <libnet.h>
+#endif
+
+
+typedef struct _InetSocketOptions
+{
+  SocketOptions super;
+  gint ttl;
+  gint tos;
+} InetSocketOptions;
+
 typedef struct _AFInetSourceDriver
 {
   AFSocketSourceDriver super;
+  InetSocketOptions sock_options;
 } AFInetSourceDriver;
 
-LogDriver *afinet_sd_new(gchar *host, gint port, guint flags);
-void afinet_sd_set_localport(LogDriver *self, gint port, gchar *service, gchar *proto);
+LogDriver *afinet_sd_new(gint af, gchar *host, gint port, guint flags);
+void afinet_sd_set_localport(LogDriver *self, gchar *service, gchar *proto);
 void afinet_sd_set_localip(LogDriver *self, gchar *ip);
 
 #define afinet_sd_set_auth(a,b)
@@ -42,12 +55,18 @@ void afinet_sd_set_localip(LogDriver *self, gchar *ip);
 typedef struct _AFInetDestDriver
 {
   AFSocketDestDriver super;
+  InetSocketOptions sock_options;
+#if ENABLE_SPOOF_SOURCE
+  gboolean spoof_source;
+  libnet_t *lnet_ctx;
+#endif
 } AFInetDestDriver;
 
-LogDriver *afinet_dd_new(gchar *host, gint port, guint flags);
-void afinet_dd_set_localport(LogDriver *self, gint port, gchar *service, gchar *proto);
-void afinet_dd_set_destport(LogDriver *self, gint port, gchar *service, gchar *proto);
+LogDriver *afinet_dd_new(gint af, gchar *host, gint port, guint flags);
+void afinet_dd_set_localport(LogDriver *self, gchar *service, gchar *proto);
+void afinet_dd_set_destport(LogDriver *self, gchar *service, gchar *proto);
 void afinet_dd_set_localip(LogDriver *self, gchar *ip);
 void afinet_dd_set_sync_freq(LogDriver *self, gint sync_freq);
+void afinet_dd_set_spoof_source(LogDriver *self, gboolean enable);
 
 #endif
