@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2003, 2004 BalaBit IT Ltd, Budapest, Hungary
+ * Copyright (c) 2002-2007 BalaBit IT Ltd, Budapest, Hungary                    
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -36,6 +36,13 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
+#if ENABLE_TCP_WRAPPER
+#include <tcpd.h>
+int allow_severity = 0;
+int deny_severity = 0;
+#endif
+
+
 typedef struct _GListenSource
 {
   GSource super;
@@ -349,6 +356,27 @@ afsocket_sd_format_persist_name(AFSocketSourceDriver *self, gboolean listener_na
 gboolean 
 afsocket_sd_process_connection(AFSocketSourceDriver *self, GSockAddr *peer_addr, gint fd)
 {
+#if ENABLE_TCP_WRAPPER
+  if (peer_addr && (peer_addr->sa.sa_family == AF_INET || peer_addr->sa.sa_family == AF_INET6))
+    {
+      struct request_info req;
+   
+      request_init(&req, RQ_DAEMON, "syslog-ng", RQ_FILE, fd, 0);
+      fromhost(&req);
+      if (hosts_access(&req) == 0)
+        {
+          gchar buf[256];
+          
+          msg_verbose("Syslog connection rejected by tcpd",
+                      evt_tag_str("from", g_sockaddr_format(peer_addr, buf, sizeof(buf))),
+                      NULL);
+          close(fd);
+          return TRUE;
+        }
+    }
+  
+#endif
+
   if (self->num_connections >= self->max_connections)
     {
       msg_error("Number of allowed concurrent connections exceeded",
@@ -418,6 +446,14 @@ afsocket_sd_kill_connection(AFSocketSourceConnection *sc)
   log_pipe_unref(&sc->super);
 }
 
+static void
+afsocket_sd_kill_connection_list(GList *list)
+{
+  GList *l;
+  for (l = list; l; l = g_list_next(l))
+    afsocket_sd_kill_connection((AFSocketSourceConnection *) l->data);
+}
+
 gboolean
 afsocket_sd_init(LogPipe *s, GlobalConfig *cfg, PersistentConfig *persist)
 {
@@ -542,7 +578,7 @@ afsocket_sd_deinit(LogPipe *s, GlobalConfig *cfg, PersistentConfig *persist)
       /* for AFSOCKET_STREAM source drivers this is a list, for
        * AFSOCKET_DGRAM this is a single connection */
       
-      persist_config_add(persist, afsocket_sd_format_persist_name(self, FALSE), self->connections, (GDestroyNotify) afsocket_sd_kill_connection);
+      persist_config_add(persist, afsocket_sd_format_persist_name(self, FALSE), self->connections, (GDestroyNotify) afsocket_sd_kill_connection_list);
     }
   self->connections = NULL;
 
@@ -586,9 +622,7 @@ afsocket_sd_notify(LogPipe *s, LogPipe *sender, gint notify_code, gpointer user_
     case NC_CLOSE:
     case NC_READ_ERROR:
       {
-        g_assert(0);
-        log_pipe_deinit(sender, NULL, NULL);
-        log_pipe_unref(sender);
+        g_assert_not_reached();
         break;
       }
     }
