@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2003, 2004 BalaBit IT Ltd, Budapest, Hungary
+ * Copyright (c) 2002-2007 BalaBit IT Ltd, Budapest, Hungary                    
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -88,7 +88,7 @@ void
 cfg_file_owner_set(GlobalConfig *self, gchar *owner)
 {
   if (!resolve_user(owner, &self->file_uid))
-    msg_notice("Error resolving user",
+    msg_error("Error resolving user",
                evt_tag_str("user", owner),
                NULL);
 }
@@ -97,7 +97,7 @@ void
 cfg_file_group_set(GlobalConfig *self, gchar *group)
 {
   if (!resolve_group(group, &self->file_gid))
-    msg_notice("Error resolving group",
+    msg_error("Error resolving group",
                evt_tag_str("group", group),
                NULL);
 }
@@ -112,7 +112,7 @@ void
 cfg_dir_owner_set(GlobalConfig *self, gchar *owner)
 {
   if (!resolve_user(owner, &self->dir_uid))
-    msg_notice("Error resolving user",
+    msg_error("Error resolving user",
                evt_tag_str("user", owner),
                NULL);
 }
@@ -121,7 +121,7 @@ void
 cfg_dir_group_set(GlobalConfig *self, gchar *group)
 {
   if (!resolve_group(group, &self->dir_gid))
-    msg_notice("Error resolving group",
+    msg_error("Error resolving group",
                evt_tag_str("group", group),
                NULL);
 }
@@ -184,11 +184,11 @@ cfg_init(GlobalConfig *cfg, PersistentConfig *persist)
   gint regerr;
   
   if (cfg->file_template_name && !(cfg->file_template = cfg_lookup_template(cfg, cfg->file_template_name)))
-    msg_notice("Error resolving file template",
+    msg_error("Error resolving file template",
                evt_tag_str("name", cfg->file_template_name),
                NULL);
   if (cfg->proto_template_name && !(cfg->proto_template = cfg_lookup_template(cfg, cfg->proto_template_name)))
-    msg_notice("Error resolving protocol template",
+    msg_error("Error resolving protocol template",
                evt_tag_str("name", cfg->proto_template_name),
                NULL);
 
@@ -252,7 +252,7 @@ cfg_new(gchar *fname)
   self->time_reopen = 60;
   self->time_reap = 60;
 
-  self->log_fifo_size = 100;
+  self->log_fifo_size = 1000;
   self->log_fetch_limit = 10;
   self->log_iw_size = 100;
   self->log_msg_size = 8192;
@@ -409,6 +409,8 @@ typedef struct _PersistentConfigEntry
 {
   gpointer value;
   GDestroyNotify destroy;
+  /* this value is a string and should be saved */
+  gboolean survive_across_restarts:1;
 } PersistentConfigEntry;
 
 static void
@@ -419,8 +421,8 @@ persist_config_entry_free(PersistentConfigEntry *self)
   g_free(self);
 }
 
-void 
-persist_config_add(PersistentConfig *self, gchar *name, gpointer value, GDestroyNotify destroy)
+static PersistentConfigEntry *
+persist_config_add_entry(PersistentConfig *self, gchar *name, gpointer value, GDestroyNotify destroy)
 {
   PersistentConfigEntry *p;
   
@@ -428,11 +430,11 @@ persist_config_add(PersistentConfig *self, gchar *name, gpointer value, GDestroy
     {
       if (g_hash_table_lookup(self->keys, name))
         {
-          msg_error("Duplicate configuration elements refer to the same persistent config, this is probably not what you want", 
+          msg_error("Internal error, duplicate configuration elements refer to the same persistent config", 
                     evt_tag_str("name", name),
                     NULL);
           destroy(value);
-          return;
+          return NULL;
         }
   
       p = g_new0(PersistentConfigEntry, 1);
@@ -441,6 +443,31 @@ persist_config_add(PersistentConfig *self, gchar *name, gpointer value, GDestroy
       p->destroy = destroy;
   
       g_hash_table_insert(self->keys, g_strdup(name), p);
+      return p;
+    }
+  return NULL;
+}
+
+void
+persist_config_add(PersistentConfig *self, gchar *name, gpointer value, GDestroyNotify destroy)
+{
+  persist_config_add_entry(self, name, value, destroy);
+}
+
+/**
+ * persist_config_add_survivor:
+ *
+ * Add a "surviving" value, a value that is remembered accross restarts.
+ **/
+void
+persist_config_add_survivor(PersistentConfig *self, gchar *name, gchar *value)
+{
+  PersistentConfigEntry *p;
+  
+  p = persist_config_add_entry(self, name, g_strdup(value), g_free);
+  if (p)
+    {
+      p->survive_across_restarts = TRUE;
     }
 }
 
@@ -464,6 +491,121 @@ persist_config_fetch(PersistentConfig *self, gchar *name)
   return res;
 }
 
+static gboolean
+persist_config_write_string(FILE *persist_file, gchar *str)
+{
+  guint32 length;
+  
+  length = htonl(strlen(str));
+  if (fwrite(&length, 1, sizeof(length), persist_file) == sizeof(length) &&
+      fwrite(str, 1, ntohl(length), persist_file) == ntohl(length))
+    {
+      return TRUE;
+    }
+  return FALSE;
+}
+
+static gboolean
+persist_config_read_string(FILE *persist_file, gchar **str)
+{
+  guint32 length;
+  
+  if (fread(&length, 1, sizeof(length), persist_file) != sizeof(length))
+    return FALSE;
+  length = ntohl(length);
+  if (length > 4096)
+    return FALSE;
+  *str = g_malloc(length + 1);
+  (*str)[length] = 0;
+  if (fread(*str, 1, length, persist_file) != length)
+    {
+      g_free(*str);
+      return FALSE;
+    }
+  return TRUE;
+}
+
+static void
+persist_config_save_value(gchar *key, PersistentConfigEntry *entry, FILE *persist_file)
+{
+  if (entry->survive_across_restarts)
+    {
+      /* NOTE: we ignore errors here, as we cannot bail out from the
+       * g_hash_table_foreach() loop anyway. */
+      
+      persist_config_write_string(persist_file, key);
+      persist_config_write_string(persist_file, (gchar *) entry->value);
+    }
+}
+
+void
+persist_config_save(PersistentConfig *self)
+{
+  FILE *persist_file;
+  
+  persist_file = fopen(PATH_PERSIST_CONFIG, "w");
+  if (persist_file)
+    {
+      if (fwrite("SLP1", 1, 4, persist_file) < 0)
+        {
+          fclose(persist_file);
+          goto error;
+        }
+      g_hash_table_foreach(self->keys, (GHFunc) persist_config_save_value, persist_file);
+      fclose(persist_file);
+      return;
+    }
+ error:
+  msg_error("Error saving persistent configuration file",
+            evt_tag_str("name", PATH_PERSIST_CONFIG),
+            NULL);
+  
+}
+
+void
+persist_config_load(PersistentConfig *self)
+{
+  FILE *persist_file;
+  
+  persist_file = fopen(PATH_PERSIST_CONFIG, "r");
+  if (persist_file)
+    {
+      gchar magic[4];
+      gchar *key, *value;
+      
+      if (fread(magic, 1, sizeof(magic), persist_file) != 4)
+        {
+          msg_error("Error loading persistent configuration file",
+                    evt_tag_str("name", PATH_PERSIST_CONFIG),
+                    NULL);
+          goto close_and_exit;
+        }
+      if (memcmp(magic, "SLP1", 4) != 0)
+        {
+          msg_error("Persistent configuration file is in invalid format", NULL);
+          goto close_and_exit;
+        }
+      while (persist_config_read_string(persist_file, &key))
+        {
+          if (persist_config_read_string(persist_file, &value))
+            {
+              /* add a non-surviving entry, thus each value is only
+               * written/read once unless the code readds it, this is needed
+               * to limit the size of the persistent configuration file */
+              
+              persist_config_add(self, key, value, g_free);
+            }
+          else
+            {
+              g_free(key);
+              break;
+            }
+        }
+ close_and_exit:
+      fclose(persist_file);
+    }
+}
+
 PersistentConfig *
 persist_config_new(void)
 {
