@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2003, 2004 BalaBit IT Ltd, Budapest, Hungary
+ * Copyright (c) 2002-2007 BalaBit IT Ltd, Budapest, Hungary                    
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -37,11 +37,13 @@ extern struct sl_name sl_levels[];
 extern struct sl_name sl_facilities[];
 
 /* returns an index where this name is found */
-int syslog_lookup_name(const char *name, struct sl_name names[]);
-char *syslog_lookup_value(int value, struct sl_name names[]);
+int syslog_name_lookup_id_by_name(const char *name, struct sl_name names[]);
+int syslog_name_lookup_value_by_name(const char *name, struct sl_name names[]);
+const char *syslog_name_lookup_name_by_value(int value, struct sl_name names[]);
+
 guint32 syslog_make_range(guint32 r1, guint32 r2);
 
-#define syslog_lookup_level(name) syslog_lookup_name(name, sl_levels)
-#define syslog_lookup_facility(name) syslog_lookup_name(name, sl_facilities)
+#define syslog_name_lookup_level_by_name(name) syslog_name_lookup_value_by_name(name, sl_levels)
+#define syslog_name_lookup_facility_by_name(name) syslog_name_lookup_id_by_name(name, sl_facilities)
 
 #endif
